import vex from 'vex-js';

import { loadIcon } from 'colby-bootstrap/js/loading';
import fitParentToChild from 'colby-bootstrap/js/fit-parent-to-child';

vex.defaultOptions.className = 'vex-theme-default';

const postsUrl = `${wpData.bloginfoUrl}/wp-json/wp/v2/posts`;

export default class ListPosts {
  constructor(settings) {
    this.settings = settings;

    this.container = document.getElementById(this.settings.id);

    if (!this.container) {
      return;
    }

    this.settings.type = this.settings.type ||
      this.container.getAttribute('data-type') ||
      'excerpt-list';
    this.settings['show-thumbnails'] = this.settings['show-thumbnails'] ||
      this.container.getAttribute('data-show-thumbnails') ||
      'true';

    if (!this.settings.id) {
      return;
    }

    this.posts = [];

    this.run();
  }

  run() {
    this.container.innerHTML = loadIcon;

    const category = this.container.getAttribute('data-category');
    const categoriesParam = category ? `?categories=${category}` : '';

    fetch(`${postsUrl}${categoriesParam}`)
      .then(response => response.json())
      .then(posts => {
        this.posts = posts;

        switch (this.settings.type) {
          case 'modal-grid':
            return this.drawModalGridPosts();
          default:
            return this.drawPosts();
        }
      });
  }

  drawPosts() {
    this.container.innerHTML = this.posts.map((post, index) => {
      let featuredImage = '';
      if (this.settings['show-thumbnails'] === 'true') {
        featuredImage = `
					<div class=list-posts__thumbnail-container>
						${post.featured_media}
					</div>
				`;
      }

      return `
				<article class=list-posts__excerpt>
					<header class=list-posts__header>
						<h2>
							<a href=${post.link}>${post.title.rendered}</a>
						</h2>
					</header>
					<section class=list-posts__main>
						${featuredImage}
						${post.excerpt.rendered}
					</section>
				</article>
			`;
    }).join('');
  }

  drawModalGridPosts() {
    this.container.innerHTML = this.posts.map(this.drawModalGridPost).join('');

    [].forEach.call(document.querySelectorAll('.modal-grid__post'), (
      element,
      index
    ) =>
      {
        element.addEventListener(
          'click',
          event => this.handlePostModal(event, element, index)
        );
      });
  }

  drawModalGridPost(post, index) {
    return `
      <a class=modal-grid__post href=${post.link}>
				<div class=modal-grid__main>
	        <h2>${post.title.rendered}</h2>
	        <p>${post.excerpt.rendered}</p>
				</div>
        <div class=modal-grid__image-container>
          ${post.featured_media}
        </div>
      </a>
    `;
  }

  handlePostModal(event, element, index) {
    event.preventDefault();

    let html = '';

    if (this.posts[index] && this.posts[index].rendered_html) {
      html = this.posts[index].rendered_html;
    }

    vex.open({ unsafeContent: html });
  }
}
