export default class Tooltips {
  constructor(settings) {
    // Make sure a settings object exists.
    settings = settings || {};

    // Defaults go here.
    this.tooltipClass = settings.tooltipClass || 'tooltip';
    this.tooltipWidth = settings.tooltipWidth
      ? Number(settings.tooltipWidth)
      : 200;

    this.setUpTooltips();

    // Above function makes these. If they are empty, don't run.
    if (this.tooltips && this.tooltipBoxes) {
      this.run();
    }
  }

  run() {
    this.tooltipBoxes.forEach(this.drawTooltip.bind(this));
    this.tooltips.forEach(this.positionTooltipBox.bind(this));
    this.tooltips.forEach(this.handleEvents.bind(this));
  }

  /**
	 * Gather all the tooltips into an array and add basic CSS.
	 */
  setUpTooltips() {
    const tooltips = document.querySelectorAll('[data-tooltip]');

    if (!tooltips) {
      return;
    }

    // Convert nodelist to array for older browsers.
    this.tooltips = [].map.call(tooltips, tooltip => tooltip);

    // Make a corresponding array of the content for the tooltip boxes.
    this.tooltipBoxes = [].map.call(
      this.tooltips,
      tooltip => tooltip.getAttribute('data-tooltip') || null
    );

    this.tooltips.forEach(tooltip => {
      // Absolute positioning used in child.
      tooltip.style.position = 'relative';

      // If the tooltip attribute is not on <a>, make the cursor act like it is.
      tooltip.style.cursor = 'pointer';
      // If the tooltip attribute is placed on something other than <a>, make
      // sure it doesn't break into a new line unless this is overridden in CSS.
      tooltip.style.display = 'inline-block';
    });

    return this.tooltips.length > 0;
  }

  /**
	 * Make a <div> containing the tooltip box content and drop it in the container.
	 */
  drawTooltip(box, key) {
    this.contentBoxes = this.contentBoxes || [];

    const contentBox = document.createElement('DIV');
    contentBox.innerHTML = box;
    contentBox.classList.add(`${this.tooltipClass}__box`);

    // Set styles.
    contentBox.style.position = 'absolute';
    contentBox.style.background = 'white';
    contentBox.style.padding = '.75em 1.5em';
    contentBox.style.border = '1px solid rgba(0, 0, 0, .15)';
    contentBox.style['z-index'] = '5555';
    contentBox.style.visibility = 'hidden';
    contentBox.style.width = `${this.tooltipWidth}px`;

    this.contentBoxes.push(contentBox);

    this.tooltips[key].appendChild(contentBox);
  }

  /**
	 * Set position values based on where the tooltip box is situated in the window.
	 */
  positionTooltipBox(tooltip, i) {
    const box = tooltip.querySelector(`.${this.tooltipClass}__box`);
    const boxRect = box.getBoundingClientRect();

    box.style.bottom = '0';

    if (boxRect.left < window.innerWidth - this.tooltipWidth) {
      box.style.left = `${tooltip.clientWidth + 10}px`;
    } else {
      box.style.right = `${tooltip.clientWidth + 10}px`;
    }
  }

  handleEvents(tooltip, i) {
    tooltip.addEventListener('mouseover', () => {
      this.contentBoxes[i].style.visibility = 'visible';
    });

    tooltip.addEventListener('mouseout', () => {
      this.contentBoxes[i].style.visibility = 'hidden';
    });
  }
}
